//
//  RMNetworkClientProtocol.h
//  RMFTP
//
//  Created by Simon Maddox on 23/08/2018.
//  Copyright © 2018 Realmac Software. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "RMFTP-Constants.h"

@class RMFTPURLCredential;

@protocol RMNetworkClientOperation <NSObject>

@property (nonatomic, readonly) NSError *error;
@property (nonatomic, copy) RMNetworkClientProgressBlock progressDidChangeBlock;

@end

@protocol RMNetworkClient <NSObject>

- (NSURL *)serverURL;

- (instancetype)initWithServerURL:(NSURL *)serverURL credentials:(RMFTPURLCredential *)credential maxConnections:(NSUInteger)maximumConnections;

- (void)connectWithCompletionHandler:(void (^)(NSError *error))completionHandler;
- (void)disconnect;
- (void)shutdown;

- (void)absolutePathAtPath:(NSString *)path completionHandler:(void (^)(NSString *absolutePath, NSError *error))completionHandler;
- (void)listDirectoryContentsAtPath:(NSString *)path completionHandler:(void (^)(NSString *absolutePath, NSArray *directories, NSArray *files, NSError *error))completionHandler;
- (void)createDirectoryAtPath:(NSString *)path named:(NSString *)directoryName completionHandler:(void (^)(NSError *error))completionHandler;
- (void)deleteFileAtPath:(NSString *)path isDirectory:(BOOL)isDirectory completionHandler:(void (^)(NSError *error))completionHandler;
- (void)downloadFileAtPath:(NSString *)path toPath:(NSString *)localPath completionHandler:(void (^)(NSError *error))completionHandler;

- (NSOperation <RMNetworkClientOperation> *)newUploadFileOperationWithlocalFileURL:(NSURL *)localFileURL toPath:(NSString *)path;
- (NSOperation <RMNetworkClientOperation> *)newUploadFileOperationWithLocalFileURL:(NSURL *)localFileURL destinationFilename:(NSString *)destinationFilename toPath:(NSString *)path renameAfterUpload:(BOOL)renameAfterUpload;

- (NSOperation *)newUploadFilesOperationFromLocalBaseURL:(NSURL *)localBaseURL filePaths:(NSArray *)filePaths toPath:(NSString *)path;


@end
